// alu.java

public class alu
{

/*
*    000000  left shift  - constant
*    000010  right shift - constant
*    000011  arithmetic right shift - constant
*    000100  left shift - register
*    000110  right shift - register
*    000111  arithmetic right shift - register
* --------------------------------------------
*    100000  add, signed (overflow)
*    100001  add, unsigned (no overflow)
*    100010  subtract, signed (overflow)
*    100011  subtract, unsigned (no overflow)
* --------------------------------------------
*    100100  AND
*    100101  OR
*    100110  XOR
*    100111  NOR
* --------------------------------------------
*    101010  slt
* --------------------------------------------
*    001000  jr
*    001100  syscall
* --------------------------------------------
*    
*    010000  mfhi
*    010010  mflo
*/
	boolean zero, overflow;
	boolean ovfl, cout;

	public int alu(int a, int b, int shift, int control)
	{
		int out;
		int adder_out, shift_out, logic_out;
		boolean subtract = (control&0x2)>0;
		adder_out = adder(a,b,subtract);
		
		int wshift = ((control&0x04)>0? a : shift) & 0x1f;
		zero = adder_out == 0;
		boolean overflow = (adder_out<0? (a<0)&&(b<0): (a>=0)&&(b>=0));
		ovfl = ((control&0x1)>0? false: overflow);

		switch ((control>>2)&0xf)
		{
			case 0:
			case 1:	out = shifter(b,control,wshift);
				break;
			case 8: out = adder_out;
				break;
			case 9: out = logic_unit(a,b,control);
				break;
			case 10: out = (adder_out<0? 1: 0); // slt
				 break;
			default:
				out = 0;
		}
		return out;
	}

	public int adder(int a, int b, boolean subtract)
	{
		int sum;
		if (subtract) sum = a - b;
		else sum = a + b;
		cout = false;
		return sum;
	}

	public int logic_unit(int a, int b, int control)
	{
		int out;
		switch (control&0x3)
		{
			case 0: out = a&b;
				break;
			case 1: out = a|b;
				break;
			case 2: out = a^b;
				break;
			case 3: out= ~(a|b);
				break;
			default:
				out = 0;
				break;
		}
		return out;
	}
/*
*     control
*      00  sll  shift left logical
*      10  srl  shift right logical
*      01  nop  no operation
*      11  sra  shift right arithmetic
*/
	public int shifter(int inp, int control, int shift)
	{
		int out;
		shift = (shift&0x1f);
		switch (control&0x3)
		{
			// left logical
			case 0: out = inp<<shift;
				break;
			// nop
			case 1: out = inp;
				break;
			// right logical
			case 2: int mask = ((-1)<<(32-shift));
				out = (inp>>shift)&(~mask);
				break;
			// right arithmetic
			case 3: out = inp>>shift;
				break;
			default:
				out = inp;
				break;
		}
		return out;
	}

	public void test()
	{
		int a, b, d;
		int shift, control;

		a = 5;
		b = 12;
		shift = 0;
		control = 0x20;

		System.out.printf("control\ta\tb\td\tshift\n");
		for (int k=0; k<4; k++)
		{
			 if (k==1) {
				a = 28;
				b = -1;
				control = 0x06;
				d = shifter(b,control,a);
			}
			else if (k==2) {
				control = 0x07;
			}
			else if (k==3) {
				a = 3;
				control = 0x04;
			}
			d = alu(a,b,shift,control);
			System.out.printf("%2x\t%d\t%d\t%d\t%d\n",control,a,b,d,shift);
		}
	}

	public static void main(String args[])
	{

		alu my_alu = new alu();
		my_alu.test();

	}
}