/*
*    000000  left shift  - constant
*    000010  right shift - constant
*    000011  arithmetic right shift - constant
*    000100  left shift - register
*    000110  right shift - register
*    000111  arithmetic right shift - register
* --------------------------------------------
*    100000  add, signed (overflow)
*    100001  add, unsigned (no overflow)
*    100010  subtract, signed (overflow)
*    100011  subtract, unsigned (no overflow)
* --------------------------------------------
*    100100  AND
*    100101  OR
*    100110  XOR
*    100111  NOR
* --------------------------------------------
*    101010  slt
* --------------------------------------------
*    001000  jr
*    001100  syscall
* --------------------------------------------
*    
*    010000  mfhi
*    010010  mflo
*/  
module alu(a,b,shift,control,out,zero,ovfl);
parameter L = 32;
input [L-1:0] a, b;
input [4:0] shift;
input [5:0] control;
output [L-1:0] out;
output zero, ovfl;
reg [L-1:0] out;

wire [4:0] wshift;
wire [31:0] shift_in, shift_out;
wire [L-1:0] logic_out, adder_out;
wire cout, overflow;

assign wshift = (control[2]? a[4:0]: shift);
assign shift_in = b;

adder #(.L(L)) alu_adder(a,b,control[1],cout,adder_out);
barrel alu_barrel(shift_in,control[1:0],wshift,shift_out);
logic_unit #(.L(L)) alu_logic(a,b,control[1:0],logic_out);

assign zero = (adder_out==0);
assign overflow  = (adder_out[L-1]? (~a[L-1])&(~b[L-1]): a[L-1]&b[L-1] );
assign ovfl = (control[0]? 1'b0: overflow);



always
	case (control[5:2])
	4'b0000: out = shift_out[L-1:0];
	//4'b0100: out = (control[1]?Lo:Hi); // used for mfhi, mflow
	4'b1010: out = adder_out[L-1]; // slt
	4'b1000: out = adder_out;
	4'b1001: out = logic_out;
	default: out = 0;
	endcase

endmodule
