module decode_opcode(opcode,
	RegDst,ALUSrc,MemtoReg,RegWrite,
	MemRead,MemWrite,Branch,ALUOp);
input [5:0] opcode;
output RegDst, ALUSrc, MemtoReg, RegWrite, MemRead, MemWrite, Branch;
output [7:0] ALUOp;
`define immed_op ALUOp[7]

wire R_format, Lw, Sw, Beq;

assign R_format = (opcode == 6'b000000);
assign Beq = (opcode == 6'b000100);

/*
*   100011  load word
*   100000  load byte, sign extend
*   100100  load byte, zero extend
*   101000  store byte
*   101011  store word
*/

assign Lw = (opcode[5]? ~opcode[3] : 1'b0);
assign Sw = (opcode[5]?  opcode[3] : 1'b0);

// immediate instructions
// ALUOp[7] flags an immediate instruction
// ALUOp[6] flags an arithmetic instruction (sign extension required)
reg [7:0] ALUOp;
always 
case (opcode)
6'b001000: ALUOp <= 8'b11100000; // add signed
6'b001001: ALUOp <= 8'b11100001; // add unsigned
6'b001010: ALUOp <= 8'b11101010; // slt
6'b001100: ALUOp <= 8'b10100100; // AND
6'b001101: ALUOp <= 8'b10100101; // OR
6'b001110: ALUOp <= 8'b10100110; // XOR
default:   ALUOp <= 8'b00xxxxxx;
endcase

assign RegDst = R_format;
assign ALUSrc = Lw | Sw;
assign MemtoReg = Lw;
assign RegWrite = R_format | Lw | `immed_op;
assign MemRead = Lw;
assign MemWrite = Sw;
assign Branch = Beq;

endmodule