module srisc(clock,reset,instruction,a,b,out,zero,ovfl,RegWrite,address);
parameter L = 31;
`define immed_op ALUOp[7]
input clock, reset;
output [31:0] instruction;

output [L-1:0] a, b;
output [L-1:0] out;
output zero, ovfl;
output RegWrite;
output [4:0] address;

wire [4:0] shift;


wire RegDst, ALUSrc, MemtoReg;
wire MemWrite, MemRead, Branch;
wire [7:0] ALUOp;
wire [5:0] func;

wire [5:0] opcode;
wire [4:0] rs,rt,rd;
wire [31:0] immed;

reg [4:0] address;

instr instr_cache(.address(address),.inclock(clock),.q(instruction));

always @(posedge clock or negedge reset)
if (!reset) address <= 5'b00000;
else address <= address + 5'b00001;


assign opcode = instruction[31:26];
assign rs = instruction[25:21];
assign rt = instruction[20:16];
assign rd = instruction[15:11];
assign shift = instruction[10:6];

assign immed[15:0] = instruction[15:0];
assign immed[31:16] = (ALUOp[6]&instruction[15]? 16'hFFFF: 16'h0000);

decode_opcode dop(opcode,
	RegDst, ALUSrc, MemtoReg, RegWrite, MemRead, MemWrite,
	Branch, ALUOp);

wire [L-1:0] bt;
wire [4:0] rdm;

assign func = (`immed_op? ALUOp[5:0]: instruction[5:0]);
assign bt = (`immed_op? immed[L-1:0] : b);
assign rdm = (`immed_op? rt : rd);
alu #(.L(L)) alu_unit(a,bt,shift,func,out,zero,ovfl);

reg_file #(.L(L)) reg_unit(clock,reset,a,b,out,rs,rt,rdm,RegWrite);


endmodule